'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var url = require('url');
var node = require('@remix-run/node');
var serverRuntime = require('@remix-run/server-runtime');

function createRequestHandler({
  build,
  getLoadContext,
  mode = process.env.NODE_ENV,
  originPaths = [],
  onError = () => {}
}) {
  let platform = {};
  let handleRequest = serverRuntime.createRequestHandler(build, platform, mode);
  const originPathRegexes = originPaths.map(s => typeof s === "string" ? new RegExp(s) : s);
  return (event, context, callback) => {
    const cloudfrontRequest = event.Records[0].cf.request;

    if (originPathRegexes.some(r => r.test(cloudfrontRequest.uri))) {
      /* Continue this work if you foresee viability of s3Origin
      const s3headers = new Set([
        "origin",
        "access-control-request-headers",
        "access-control-request-method",
        "accept-encoding",
        "content-length",
        "if-modified-since",
        "if-none-match",
        "if-range",
        "if-unmodified-since",
        "transfer-encoding",
        "via",
      ]);
      Object.keys(cloudfrontRequest.headers).forEach(k => {
        if (!s3headers.has(k)) {
          delete cloudfrontRequest.headers[k];
        }
      });
      */
      context.callbackWaitsForEmptyEventLoop = false;
      return callback(undefined, cloudfrontRequest);
    }

    let request = createRemixRequest(event);
    let loadContext = typeof getLoadContext === "function" ? getLoadContext(event) : undefined;
    return handleRequest(request, loadContext).then(async response => ({
      status: String(response.status),
      headers: createCloudFrontHeaders(response.headers),
      bodyEncoding: "text",
      body: await response.text()
    })).catch(e => {
      console.error("Remix failed to handle request:");
      console.error(e);
      onError(e);
      return {
        status: "500",
        headers: {},
        bodyEncoding: "text",
        body: e.message
      };
    });
  };
}
function createCloudFrontHeaders(responseHeaders) {
  let headers = {};
  let rawHeaders = responseHeaders.raw();

  for (let key in rawHeaders) {
    let value = rawHeaders[key];

    for (let v of value) {
      headers[key] = [...(headers[key] || []), {
        key,
        value: v
      }];
    }
  }

  return headers;
}
function createRemixHeaders(requestHeaders) {
  let headers = new node.Headers();

  for (let [key, values] of Object.entries(requestHeaders)) {
    for (let {
      value
    } of values) {
      if (value) {
        headers.append(key, value);
      }
    }
  }

  return headers;
}
function createRemixRequest(event) {
  var _request$body;

  let request = event.Records[0].cf.request;
  let host = request.headers["host"] ? request.headers["host"][0].value : undefined;
  let search = request.querystring.length ? `?${request.querystring}` : "";
  let url$1 = new url.URL(request.uri + search, `https://${host}`);
  return new node.Request(url$1.toString(), {
    method: request.method,
    headers: createRemixHeaders(request.headers),
    body: (_request$body = request.body) !== null && _request$body !== void 0 && _request$body.data ? request.body.encoding === "base64" ? Buffer.from(request.body.data, "base64").toString() : request.body.data : undefined
  });
}

exports.createCloudFrontHeaders = createCloudFrontHeaders;
exports.createRemixHeaders = createRemixHeaders;
exports.createRemixRequest = createRemixRequest;
exports.createRequestHandler = createRequestHandler;
